@@ -18,6 +18,8 @@ from openmdao.components.demux_comp import DemuxComp
 from openmdao.components.dot_product_comp import DotProductComp
 from openmdao.components.eq_constraint_comp import EQConstraintComp
 from openmdao.components.exec_comp import ExecComp
+from openmdao.components.explicit_func_comp import ExplicitFuncComp
+from openmdao.components.implicit_func_comp import ImplicitFuncComp
 from openmdao.components.external_code_comp import ExternalCodeComp
 from openmdao.components.external_code_comp import ExternalCodeImplicitComp
 from openmdao.components.ks_comp import KSComp
@@ -91,6 +93,11 @@ from openmdao.recorders.case_reader import CaseReader
 from openmdao.visualization.n2_viewer.n2_viewer import n2
 from openmdao.visualization.connection_viewer.viewconns import view_connections
 from openmdao.visualization.partial_deriv_plot import partial_deriv_plot
+from openmdao.visualization.timing_viewer.timer import timing_context
+from openmdao.visualization.timing_viewer.timing_viewer import view_timing, view_timing_dump, \
+    view_MPI_timing
+from openmdao.visualization.options_widget import OptionsWidget
+from openmdao.visualization.case_viewer.case_viewer import CaseViewer
 
 # Notebook Utils
 from openmdao.utils.notebook_utils import notebook_mode, display_source, show_options_table, cite
@@ -105,16 +112,34 @@ from openmdao.utils.om_warnings import issue_warning, reset_warnings, OpenMDAOWa
     MPIWarning, UnitsWarning, SolverWarning, OMDeprecationWarning, \
     OMInvalidCheckDerivativesOptionsWarning
 
+# Reports System
+from openmdao.utils.reports_system import register_report, get_reports_dir, list_reports, \
+    clear_reports
 
-from openmdao.utils.general_utils import wing_dbg
+import os
+import builtins
+
+from openmdao.utils.general_utils import wing_dbg, env_truthy
+from openmdao.utils.array_utils import shape_to_len
+
+wing_dbg()
 
 # set up tracing or memory profiling if env vars are set.
-import os
-if os.environ.get('OPENMDAO_TRACE'):
+if env_truthy('OPENMDAO_TRACE'):
     from openmdao.devtools.itrace import setup, start
     setup(os.environ['OPENMDAO_TRACE'])
     start()
-elif os.environ.get('OPENMDAO_PROF_MEM'):
+elif env_truthy('OPENMDAO_PROF_MEM'):
     from openmdao.devtools.iprof_mem import setup, start
     setup(os.environ['OPENMDAO_PROF_MEM'])
     start()
+
+
+if env_truthy('FLUSH_PRINT'):
+    _oldprint = builtins.print
+
+    def _flushprint(*args, **kwargs):
+        kwargs['flush'] = True
+        _oldprint(*args, **kwargs)
+
+    builtins.print = _flushprint
