@@ -21,7 +21,7 @@ from openmdao.utils.general_utils import set_pyoptsparse_opt
 from openmdao.utils.array_utils import array_viz
 from openmdao.utils.coloring import _compute_coloring, compute_total_coloring, Coloring
 from openmdao.utils.mpi import MPI
-from openmdao.utils.testing_utils import use_tempdirs
+from openmdao.utils.testing_utils import use_tempdirs, set_env_vars
 from openmdao.test_suite.tot_jac_builder import TotJacBuilder
 from openmdao.utils.general_utils import run_driver
 
@@ -90,7 +90,7 @@ class DynPartialsComp(om.ExplicitComponent):
 
 def run_opt(driver_class, mode, assemble_type=None, color_info=None, derivs=True,
             recorder=None, has_lin_constraint=True, has_diag_partials=True, partial_coloring=False,
-            use_vois=True, auto_ivc=False, con_alias=False, **options):
+            use_vois=True, auto_ivc=False, con_alias=False, check=False, **options):
 
     p = om.Problem(model=CounterGroup())
 
@@ -248,7 +248,7 @@ def run_opt(driver_class, mode, assemble_type=None, color_info=None, derivs=True
     if recorder:
         p.driver.add_recorder(recorder)
 
-    p.setup(mode=mode, derivatives=derivs)
+    p.setup(mode=mode, derivatives=derivs, check=check)
     if use_vois:
         p.run_driver()
     else:
@@ -262,8 +262,11 @@ class SimulColoringPyoptSparseTestCase(unittest.TestCase):
 
     def setUp(self):
         # make sure no default reports run because they'll mess up run counts
-        om.clear_reports() 
+        om.clear_reports()
 
+    # turn off TESTFLO_RUNNING so that reports will be active, in order to detect a bug
+    # when scaling report and coloring are both active.
+    @set_env_vars(TESTFLO_RUNNING='0', OPENMDAO_REPORTS='scaling')
     @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
     def test_dynamic_total_coloring_snopt_auto(self):
         # first, run w/o coloring
@@ -576,7 +579,7 @@ class SimulColoringPyoptSparseRevTestCase(unittest.TestCase):
     """Reverse coloring tests for pyoptsparse."""
     def setUp(self):
         # make sure no default reports run because they'll mess up run counts
-        om.clear_reports() 
+        om.clear_reports()
 
     @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
     def test_dynamic_rev_simul_coloring_snopt(self):
@@ -639,7 +642,7 @@ class SimulColoringScipyTestCase(unittest.TestCase):
 
     def setUp(self):
         # make sure no default reports run because they'll mess up run counts
-        om.clear_reports() 
+        om.clear_reports()
 
     def test_bad_mode(self):
         p_color_fwd = run_opt(om.ScipyOptimizeDriver, 'fwd', optimizer='SLSQP', disp=False, dynamic_total_coloring=True)
@@ -898,7 +901,7 @@ class SimulColoringRevScipyTestCase(unittest.TestCase):
 
     def setUp(self):
         # make sure no default reports run because they'll mess up run counts
-        om.clear_reports() 
+        om.clear_reports()
 
     def test_summary(self):
         p_color = run_opt(om.ScipyOptimizeDriver, 'auto', optimizer='SLSQP', disp=False, dynamic_total_coloring=True)
