@@ -9,7 +9,7 @@ additional MPI capability.
 import sys
 import json
 import signal
-from distutils.version import LooseVersion
+from packaging.version import Version
 
 import numpy as np
 from scipy.sparse import coo_matrix
@@ -201,6 +201,7 @@ class pyOptSparseDriver(Driver):
         self._in_user_function = False
         self._check_jac = False
         self._exc_info = None
+        self._total_jac_format = 'dict'
 
         self.cite = CITATIONS
 
@@ -216,12 +217,6 @@ class pyOptSparseDriver(Driver):
                              desc='Print the opt problem summary before running the optimization')
         self.options.declare('print_results', types=bool, default=True,
                              desc='Print pyOpt results if True')
-        self.options.declare('gradient method', default='openmdao',
-                             values={'openmdao', 'pyopt_fd', 'snopt_fd'},
-                             desc='Finite difference implementation to use',
-                             deprecation=(f"'gradient method' is not a valid python name and will "
-                                          "raise an exception in a future release.  Use "
-                                          "'gradient_method' instead.", 'gradient_method'))
         self.options.declare('gradient_method', default='openmdao',
                              values={'openmdao', 'pyopt_fd', 'snopt_fd'},
                              desc='Finite difference implementation to use')
@@ -238,14 +233,6 @@ class pyOptSparseDriver(Driver):
         self.options.declare('singular_jac_tol', default=1e-16,
                              desc='Tolerance for zero row/column check.')
 
-        # Deprecated option
-        self.options.declare('user_teriminate_signal', default=None, allow_none=True,
-                             desc='OS signal that triggers a clean user-termination. Only SNOPT'
-                             'supports this option.',
-                             deprecation=("The option 'user_teriminate_signal' was misspelled and "
-                                          "will be deprecated. Please use 'user_terminate_signal' "
-                                          "instead.", 'user_terminate_signal'))
-
     def _setup_driver(self, problem):
         """
         Prepare the driver for execution.
@@ -270,11 +257,6 @@ class pyOptSparseDriver(Driver):
 
         self._setup_tot_jac_sparsity()
 
-        # Handle deprecated option.
-        if self.options._dict['user_teriminate_signal']['val'] is not None:
-            self.options['user_terminate_signal'] = \
-                self.options._dict['user_teriminate_signal']['val']
-
     def run(self):
         """
         Excute pyOptsparse.
@@ -314,21 +296,7 @@ class pyOptSparseDriver(Driver):
             self.iter_count += 1
 
         # compute dynamic simul deriv coloring or just sparsity if option is set
-        if c_mod._use_total_sparsity:
-            coloring = None
-            if self._coloring_info['coloring'] is None and self._coloring_info['dynamic']:
-                coloring = c_mod.dynamic_total_coloring(self, run_model=not model_ran,
-                                                        fname=self._get_total_coloring_fname())
-
-            if coloring is not None:
-                # if the improvement wasn't large enough, don't use coloring
-                pct = coloring._solves_info()[-1]
-                info = self._coloring_info
-                if info['min_improve_pct'] > pct:
-                    info['coloring'] = info['static'] = None
-                    msg = f"Coloring was deactivated.  Improvement of {pct:.1f}% was less " \
-                          f"than min allowed ({info['min_improve_pct']:.1f}%)."
-                    issue_warning(msg, prefix=self.msginfo, category=DerivativesWarning)
+        coloring = self._get_coloring(run_model=not model_ran)
 
         comm = None if isinstance(problem.comm, FakeComm) else problem.comm
         opt_prob = Optimization(self.options['title'], WeakMethodWrapper(self, '_objfunc'),
@@ -346,7 +314,7 @@ class pyOptSparseDriver(Driver):
                                  lower=meta['lower'], upper=meta['upper'])
 
         if not hasattr(pyoptsparse, '__version__') or \
-           LooseVersion(pyoptsparse.__version__) < LooseVersion('2.5.1'):
+           Version(pyoptsparse.__version__) < Version('2.5.1'):
             opt_prob.finalizeDesignVariables()
         else:
             opt_prob.finalize()
@@ -360,7 +328,8 @@ class pyOptSparseDriver(Driver):
         # Calculate and save derivatives for any linear constraints.
         lcons = [key for (key, con) in con_meta.items() if con['linear']]
         if len(lcons) > 0:
-            _lin_jacs = self._compute_totals(of=lcons, wrt=indep_list, return_format='dict')
+            _lin_jacs = self._compute_totals(of=lcons, wrt=indep_list,
+                                             return_format=self._total_jac_format)
             # convert all of our linear constraint jacs to COO format. Otherwise pyoptsparse will
             # do it for us and we'll end up with a fully dense COO matrix and very slow evaluation
             # of linear constraints!
@@ -668,7 +637,7 @@ class pyOptSparseDriver(Driver):
                 self._in_user_function = True
                 sens_dict = self._compute_totals(of=self._quantities,
                                                  wrt=self._indep_list,
-                                                 return_format='dict')
+                                                 return_format=self._total_jac_format)
 
                 # First time through, check for zero row/col.
                 if self._check_jac:
