@@ -3,7 +3,7 @@ OpenMDAO Wrapper for the scipy.optimize.minimize family of local optimizers.
 """
 
 import sys
-from distutils.version import LooseVersion
+from packaging.version import Version
 
 import numpy as np
 from scipy import __version__ as scipy_version
@@ -19,7 +19,7 @@ from openmdao.utils.om_warnings import issue_warning, DerivativesWarning
 # Optimizers in scipy.minimize
 _optimizers = {'Nelder-Mead', 'Powell', 'CG', 'BFGS', 'Newton-CG', 'L-BFGS-B',
                'TNC', 'COBYLA', 'SLSQP'}
-if LooseVersion(scipy_version) >= LooseVersion("1.1"):  # Only available in newer versions
+if Version(scipy_version) >= Version("1.1"):  # Only available in newer versions
     _optimizers.add('trust-constr')
 
 # For 'basinhopping' and 'shgo' gradients are used only in the local minimization
@@ -32,7 +32,7 @@ _constraint_optimizers = {'COBYLA', 'SLSQP', 'trust-constr', 'shgo'}
 _constraint_grad_optimizers = _gradient_optimizers & _constraint_optimizers
 _eq_constraint_optimizers = {'SLSQP', 'trust-constr'}
 _global_optimizers = {'differential_evolution', 'basinhopping'}
-if LooseVersion(scipy_version) >= LooseVersion("1.2"):  # Only available in newer versions
+if Version(scipy_version) >= Version("1.2"):  # Only available in newer versions
     _global_optimizers |= {'shgo', 'dual_annealing'}
 
 # Global optimizers and optimizers in minimize
@@ -149,6 +149,7 @@ class ScipyOptimizeDriver(Driver):
         self.iter_count = 0
         self._check_jac = False
         self._exc_info = None
+        self._total_jac_format = 'array'
 
         self.cite = CITATIONS
 
@@ -400,7 +401,7 @@ class ScipyOptimizeDriver(Driver):
             # precalculate gradients of linear constraints
             if lincons:
                 self._lincongrad_cache = self._compute_totals(of=lincons, wrt=self._dvlist,
-                                                              return_format='array')
+                                                              return_format=self._total_jac_format)
             else:
                 self._lincongrad_cache = None
 
@@ -422,20 +423,7 @@ class ScipyOptimizeDriver(Driver):
             hess = None
 
         # compute dynamic simul deriv coloring if option is set
-        if coloring_mod._use_total_sparsity:
-            if ((self._coloring_info['coloring'] is None and self._coloring_info['dynamic'])):
-                coloring_mod.dynamic_total_coloring(self, run_model=False,
-                                                    fname=self._get_total_coloring_fname())
-
-                # if the improvement wasn't large enough, turn coloring off
-                info = self._coloring_info
-                if info['coloring'] is not None:
-                    pct = info['coloring']._solves_info()[-1]
-                    if info['min_improve_pct'] > pct:
-                        info['coloring'] = info['static'] = None
-                        msg = f"Coloring was deactivated.  Improvement of {pct:.1f}% was less " \
-                              f"than min allowed ({info['min_improve_pct']:.1f}%)."
-                        issue_warning(msg, prefix=self.msginfo, category=DerivativesWarning)
+        coloring = self._get_coloring(run_model=False)
 
         # optimize
         try:
@@ -698,7 +686,7 @@ class ScipyOptimizeDriver(Driver):
         """
         try:
             grad = self._compute_totals(of=self._obj_and_nlcons, wrt=self._dvlist,
-                                        return_format='array')
+                                        return_format=self._total_jac_format)
             self._grad_cache = grad
 
             # First time through, check for zero row/col.
