@@ -4,9 +4,9 @@ Class definition for SqliteRecorder, which provides dictionary backed by SQLite.
 
 from copy import deepcopy
 from io import BytesIO
-from collections import OrderedDict
 
 import os
+import gc
 import sqlite3
 from itertools import chain
 
@@ -14,21 +14,35 @@ import json
 import numpy as np
 
 import pickle
+import zlib
 
-from openmdao.recorders.case_recorder import CaseRecorder
+from openmdao import __version__ as openmdao_version
+from openmdao.recorders.case_recorder import CaseRecorder, PICKLE_VER
 from openmdao.utils.mpi import MPI
 from openmdao.utils.record_util import dict_to_structured_array
 from openmdao.utils.options_dictionary import OptionsDictionary
-from openmdao.utils.general_utils import simple_warning, make_serializable
+from openmdao.utils.general_utils import make_serializable, default_noraise
 from openmdao.core.driver import Driver
 from openmdao.core.system import System
 from openmdao.core.problem import Problem
 from openmdao.solvers.solver import Solver
+from openmdao.utils.om_warnings import issue_warning, CaseRecorderWarning
 
 
 """
 SQL case database version history.
 ----------------------------------
+14-- OpenMDAO 3.8.1
+     Metadata pickle and JSON blobs are compressed.
+     Save metadata separately for parallel runs.
+13-- OpenMDAO 3.8.1
+     Added OpenMDAO version number to recorder file
+12-- OpenMDAO 3.6.1
+     Change key for system metadata to use non-ambiguous separator
+11-- OpenMDAO 3.2
+     IndepVarComps are created automatically, so this changes some bookkeeping.
+10-- OpenMDAO 3.0
+     Added abs_err and rel_err recording to Problem recording
 9 -- OpenMDAO 3.0
      Changed the character to split the derivatives from 'of,wrt' to 'of!wrt' to allow for commas
      in variable names
@@ -50,7 +64,10 @@ SQL case database version history.
 1 -- Through OpenMDAO 2.3
      Original implementation.
 """
-format_version = 9
+format_version = 14
+
+# separator, cannot be a legal char for names
+META_KEY_SEP = '!'
 
 
 def array_to_blob(array):
@@ -60,7 +77,7 @@ def array_to_blob(array):
     Convert a numpy array to something that can be written
     to a BLOB field in sqlite.
 
-    TODO: move this to a util file?
+    TODO : move this to a util file?
 
     Parameters
     ----------
@@ -69,9 +86,8 @@ def array_to_blob(array):
 
     Returns
     -------
-    blob :
+    blob
         The blob created from the array.
-
     """
     out = BytesIO()
     np.save(out, array)
@@ -83,7 +99,7 @@ def blob_to_array(blob):
     """
     Convert sqlite BLOB to numpy array.
 
-    TODO: move this to a util file?
+    TODO : move this to a util file?
 
     Parameters
     ----------
@@ -92,7 +108,7 @@ def blob_to_array(blob):
 
     Returns
     -------
-    array :
+    array
         The array created from the blob.
     """
     out = BytesIO(blob)
@@ -104,12 +120,27 @@ class SqliteRecorder(CaseRecorder):
     """
     Recorder that saves cases in a sqlite db.
 
+    Parameters
+    ----------
+    filepath : str
+        Path to the recorder file.
+    append : bool, optional
+        Optional. If True, append to an existing case recorder file.
+    pickle_version : int, optional
+        The pickle protocol version to use when pickling metadata.
+    record_viewer_data : bool, optional
+        If True, record data needed for visualization.
+
     Attributes
     ----------
     _record_viewer_data : bool
         Flag indicating whether to record data needed to generate N2 diagram.
     connection : sqlite connection object
         Connection to the sqlite3 database.
+    metadata_connection : sqlite connection object
+        Connection to the sqlite3 database, if metadata is recorded separately.
+    _record_metadata : Whether this process is recording metadata. Always True
+        for serial runs, only True for rank 0 of parallel runs.
     _abs2prom : {'input': dict, 'output': dict}
         Dictionary mapping absolute names to promoted names.
     _prom2abs : {'input': dict, 'output': dict}
@@ -123,29 +154,20 @@ class SqliteRecorder(CaseRecorder):
         Path to the recorder file.
     _database_initialized : bool
         Flag indicating whether or not the database has been initialized.
-    _record_on_proc : bool
-        Flag indicating whether to record on this processor when running in parallel.
+    _started : set
+        set of recording requesters for which this recorder has been started.
     """
 
-    def __init__(self, filepath, append=False, pickle_version=2, record_viewer_data=True):
+    def __init__(self, filepath, append=False, pickle_version=PICKLE_VER, record_viewer_data=True):
         """
         Initialize the SqliteRecorder.
-
-        Parameters
-        ----------
-        filepath : str
-            Path to the recorder file.
-        append : bool, optional
-            Optional. If True, append to an existing case recorder file.
-        pickle_version : int, optional
-            The pickle protocol version to use when pickling metadata.
-        record_viewer_data : bool, optional
-            If True, record data needed for visualization.
         """
         if append:
             raise NotImplementedError("Append feature not implemented for SqliteRecorder")
 
         self.connection = None
+        self.metadata_connection = None
+        self._record_metadata = True
         self._record_viewer_data = record_viewer_data
 
         self._abs2prom = {'input': {}, 'output': {}}
@@ -154,43 +176,64 @@ class SqliteRecorder(CaseRecorder):
         self._pickle_version = pickle_version
         self._filepath = filepath
         self._database_initialized = False
+        self._started = set()
 
-        # default to record on all procs when running in parallel
-        self._record_on_proc = True
+        super().__init__(record_viewer_data)
 
-        super(SqliteRecorder, self).__init__(record_viewer_data)
-
-    def _initialize_database(self):
+    def _initialize_database(self, comm):
         """
         Initialize the database.
+
+        Parameters
+        ----------
+        comm : MPI.Comm or <FakeComm> or None
+            The communicator for the recorder (should be the comm for the Problem).
         """
-        if MPI:
-            rank = MPI.COMM_WORLD.rank
-            if self._parallel and self._record_on_proc:
-                filepath = '%s_%d' % (self._filepath, rank)
-                print("Note: SqliteRecorder is running on multiple processors. "
-                      "Cases from rank %d are being written to %s." %
-                      (rank, filepath))
-            elif rank == 0:
-                filepath = self._filepath
-            else:
-                filepath = None
+        filepath = None
+
+        if MPI and comm and comm.size > 1:
+            if self._record_on_proc:
+                if not self._parallel:
+                    # recording only on this proc
+                    filepath = self._filepath
+                else:
+                    # recording on multiple procs, so a separate file for each recording proc
+                    # plus a file for the common metadata, written by the lowest recording rank
+                    rank = comm.rank
+                    filepath = f"{self._filepath}_{rank}"
+                    print("Note: SqliteRecorder is running on multiple processors. "
+                          f"Cases from rank {rank} are being written to {filepath}.")
+                    if rank == min(self._recording_ranks):
+                        metadata_filepath = f'{self._filepath}_meta'
+                        print("Note: Metadata is being recorded separately as "
+                              f"{metadata_filepath}.")
+                        try:
+                            rc = os.remove(metadata_filepath)
+                            issue_warning("The existing case recorder metadata file, "
+                                          f"{metadata_filepath}, is being overwritten.",
+                                          category=UserWarning)
+                        except OSError:
+                            pass
+                        self.metadata_connection = sqlite3.connect(metadata_filepath)
+                    else:
+                        self._record_metadata = False
         else:
+            # no MPI or comm size == 1
             filepath = self._filepath
 
         if filepath:
             try:
                 os.remove(filepath)
+                issue_warning(f'The existing case recorder file, {filepath},'
+                              ' is being overwritten.', category=UserWarning)
             except OSError:
                 pass
 
             self.connection = sqlite3.connect(filepath)
-            with self.connection as c:
-                c.execute("CREATE TABLE metadata(format_version INT, "
-                          "abs2prom TEXT, prom2abs TEXT, abs2meta TEXT, var_settings TEXT)")
-                c.execute("INSERT INTO metadata(format_version, abs2prom, prom2abs) "
-                          "VALUES(?,?,?)", (format_version, None, None))
+            if self._record_metadata and self.metadata_connection is None:
+                self.metadata_connection = self.connection
 
+            with self.connection as c:
                 # used to keep track of the order of the case records across all case tables
                 c.execute("CREATE TABLE global_iterations(id INTEGER PRIMARY KEY, "
                           "record_type TEXT, rowid INT, source TEXT)")
@@ -206,7 +249,7 @@ class SqliteRecorder(CaseRecorder):
                 c.execute("CREATE TABLE problem_cases(id INTEGER PRIMARY KEY, "
                           "counter INT, case_name TEXT, timestamp REAL, "
                           "success INT, msg TEXT, inputs TEXT, outputs TEXT, residuals TEXT, "
-                          "jacobian BLOB)")
+                          "jacobian BLOB, abs_err REAL, rel_err REAL)")
                 c.execute("CREATE INDEX prob_name_ind on problem_cases(case_name)")
 
                 c.execute("CREATE TABLE system_iterations(id INTEGER PRIMARY KEY, "
@@ -220,14 +263,24 @@ class SqliteRecorder(CaseRecorder):
                           "solver_inputs TEXT, solver_output TEXT, solver_residuals TEXT)")
                 c.execute("CREATE INDEX solv_iter_ind on solver_iterations(iteration_coordinate)")
 
-                c.execute("CREATE TABLE driver_metadata(id TEXT PRIMARY KEY, "
-                          "model_viewer_data TEXT)")
-                c.execute("CREATE TABLE system_metadata(id TEXT PRIMARY KEY, "
-                          "scaling_factors BLOB, component_metadata BLOB)")
-                c.execute("CREATE TABLE solver_metadata(id TEXT PRIMARY KEY, "
-                          "solver_options BLOB, solver_class TEXT)")
+            if self._record_metadata:
+                with self.metadata_connection as m:
+                    m.execute("CREATE TABLE metadata(format_version INT, openmdao_version TEXT, "
+                              "abs2prom BLOB, prom2abs BLOB, abs2meta BLOB, var_settings BLOB,"
+                              "conns BLOB)")
+                    m.execute("INSERT INTO metadata(format_version, openmdao_version, abs2prom,"
+                              " prom2abs) VALUES(?,?,?,?)", (format_version, openmdao_version,
+                                                             None, None))
+                    m.execute("CREATE TABLE driver_metadata(id TEXT PRIMARY KEY, "
+                              "model_viewer_data TEXT)")
+                    m.execute("CREATE TABLE system_metadata(id TEXT PRIMARY KEY, "
+                              "scaling_factors BLOB, component_metadata BLOB)")
+                    m.execute("CREATE TABLE solver_metadata(id TEXT PRIMARY KEY, "
+                              "solver_options BLOB, solver_class TEXT)")
 
         self._database_initialized = True
+        if MPI and comm and comm.size > 1:
+            comm.barrier()
 
     def _cleanup_abs2meta(self):
         """
@@ -258,7 +311,7 @@ class SqliteRecorder(CaseRecorder):
                 var_settings[name][prop] = make_serializable(var_settings[name][prop])
         return var_settings
 
-    def startup(self, recording_requester):
+    def startup(self, recording_requester, comm=None):
         """
         Prepare for a new run and create/update the abs2prom and prom2abs variables.
 
@@ -266,25 +319,31 @@ class SqliteRecorder(CaseRecorder):
         ----------
         recording_requester : object
             Object to which this recorder is attached.
+        comm : MPI.Comm or <FakeComm> or None
+            The MPI communicator for the recorder (should be the comm for the Problem).
         """
-        super(SqliteRecorder, self).startup(recording_requester)
+        # we only want to set up recording once for each recording_requester
+        if recording_requester in self._started:
+            return
+
+        super().startup(recording_requester, comm)
 
         if not self._database_initialized:
-            self._initialize_database()
+            self._initialize_database(comm)
 
-        driver = None
-
-        # grab the system
+        # grab the system and driver
         if isinstance(recording_requester, Driver):
             system = recording_requester._problem().model
             driver = recording_requester
         elif isinstance(recording_requester, System):
             system = recording_requester
+            driver = None
         elif isinstance(recording_requester, Problem):
             system = recording_requester.model
             driver = recording_requester.driver
         elif isinstance(recording_requester, Solver):
             system = recording_requester._system()
+            driver = None
         else:
             raise ValueError('Driver encountered a recording_requester it cannot handle'
                              ': {0}'.format(recording_requester))
@@ -294,36 +353,29 @@ class SqliteRecorder(CaseRecorder):
         if self.connection:
 
             if driver is None:
-                desvars = system.get_design_vars(True, get_sizes=False)
+                desvars = system.get_design_vars(True, get_sizes=False, use_prom_ivc=False)
                 responses = system.get_responses(True, get_sizes=False)
-                objectives = OrderedDict()
-                constraints = OrderedDict()
+                constraints = {}
+                objectives = {}
                 for name, data in responses.items():
                     if data['type'] == 'con':
                         constraints[name] = data
                     else:
                         objectives[name] = data
             else:
-                desvars = driver._designvars
-                constraints = driver._cons
-                objectives = driver._objs
-                responses = driver._responses
+                desvars = driver._designvars.copy()
+                responses = driver._responses.copy()
+                constraints = driver._cons.copy()
+                objectives = driver._objs.copy()
 
-            inputs = system._var_allprocs_abs_names['input'] + \
-                system._var_allprocs_abs_names_discrete['input']
-
-            outputs = system._var_allprocs_abs_names['output'] + \
-                system._var_allprocs_abs_names_discrete['output']
+            inputs = list(system.abs_name_iter('input', local=False, discrete=True))
+            outputs = list(system.abs_name_iter('output', local=False, discrete=True))
 
             var_order = system._get_vars_exec_order(inputs=True, outputs=True)
 
-            full_var_set = [(outputs, 'output'),
-                            (desvars, 'desvar'), (responses, 'response'),
-                            (objectives, 'objective'), (constraints, 'constraint')]
-
             # merge current abs2prom and prom2abs with this system's version
-            self._abs2prom['input'].update(system._var_abs2prom['input'])
-            self._abs2prom['output'].update(system._var_abs2prom['output'])
+            self._abs2prom['input'].update(system._var_allprocs_abs2prom['input'])
+            self._abs2prom['output'].update(system._var_allprocs_abs2prom['output'])
             for v, abs_names in system._var_allprocs_prom2abs_list['input'].items():
                 if v not in self._prom2abs['input']:
                     self._prom2abs['input'][v] = abs_names
@@ -337,41 +389,64 @@ class SqliteRecorder(CaseRecorder):
 
             # absolute pathname to metadata mappings for continuous & discrete variables
             # discrete mapping is sub-keyed on 'output' & 'input'
-            real_meta = system._var_allprocs_abs2meta
-            disc_meta = system._var_allprocs_discrete
+            real_meta_in = system._var_allprocs_abs2meta['input']
+            real_meta_out = system._var_allprocs_abs2meta['output']
+            disc_meta_in = system._var_allprocs_discrete['input']
+            disc_meta_out = system._var_allprocs_discrete['output']
+
+            full_var_set = [(outputs, 'output'),
+                            (desvars, 'desvar'), (responses, 'response'),
+                            (objectives, 'objective'), (constraints, 'constraint')]
 
             for var_set, var_type in full_var_set:
                 for name in var_set:
-                    if name not in self._abs2meta:
-                        try:
-                            self._abs2meta[name] = real_meta[name].copy()
-                        except KeyError:
-                            self._abs2meta[name] = disc_meta['output'][name].copy()
-                        self._abs2meta[name]['type'] = []
-                        self._abs2meta[name]['explicit'] = name not in states
 
-                    if var_type not in self._abs2meta[name]['type']:
-                        self._abs2meta[name]['type'].append(var_type)
+                    # Design variables, constraints and objectives can be requested by input name.
+                    if var_type != 'output':
+                        srcname = var_set[name]['source']
+                    else:
+                        srcname = name
+
+                    if srcname not in self._abs2meta:
+                        if srcname in real_meta_out:
+                            self._abs2meta[srcname] = real_meta_out[srcname].copy()
+                        elif srcname in disc_meta_out:
+                            self._abs2meta[srcname] = disc_meta_out[srcname].copy()
+                        elif name in system._responses:
+                            for io in self._prom2abs:
+                                if srcname in self._prom2abs[io]:
+                                    abs_in = self._prom2abs[io][srcname][0]
+                                    self._abs2meta[srcname] = real_meta_in[abs_in].copy()
+                                    break
+                        self._abs2meta[srcname]['type'] = []
+                        self._abs2meta[srcname]['explicit'] = srcname not in states
+
+                    if var_type not in self._abs2meta[srcname]['type']:
+                        self._abs2meta[srcname]['type'].append(var_type)
 
             for name in inputs:
                 try:
-                    self._abs2meta[name] = real_meta[name].copy()
+                    self._abs2meta[name] = real_meta_in[name].copy()
                 except KeyError:
-                    self._abs2meta[name] = disc_meta['input'][name].copy()
+                    self._abs2meta[name] = disc_meta_in[name].copy()
                 self._abs2meta[name]['type'] = ['input']
                 self._abs2meta[name]['explicit'] = True
 
             # merge current abs2meta with this system's version
             for name, meta in self._abs2meta.items():
-                if name in system._var_abs2meta:
-                    meta.update(system._var_abs2meta[name])
+                for io in ('input', 'output'):
+                    if name in system._var_allprocs_abs2meta[io]:
+                        meta.update(system._var_allprocs_abs2meta[io][name])
+                        break
 
             self._cleanup_abs2meta()
 
             # store the updated abs2prom and prom2abs
-            abs2prom = json.dumps(self._abs2prom)
-            prom2abs = json.dumps(self._prom2abs)
-            abs2meta = json.dumps(self._abs2meta)
+            abs2prom = zlib.compress(json.dumps(self._abs2prom).encode('ascii'))
+            prom2abs = zlib.compress(json.dumps(self._prom2abs).encode('ascii'))
+            abs2meta = zlib.compress(json.dumps(self._abs2meta).encode('ascii'))
+            conns = zlib.compress(json.dumps(
+                system._problem_meta['model_ref']()._conn_global_abs_in2out).encode('ascii'))
 
             var_settings = {}
             var_settings.update(desvars)
@@ -379,25 +454,36 @@ class SqliteRecorder(CaseRecorder):
             var_settings.update(constraints)
             var_settings = self._cleanup_var_settings(var_settings)
             var_settings['execution_order'] = var_order
-            var_settings_json = json.dumps(var_settings)
+            var_settings_json = zlib.compress(
+                json.dumps(var_settings, default=default_noraise).encode('ascii'))
 
-            with self.connection as c:
-                c.execute("UPDATE metadata SET abs2prom=?, prom2abs=?, abs2meta=?, var_settings=?",
-                          (abs2prom, prom2abs, abs2meta, var_settings_json))
+            if self._record_metadata:
+                with self.metadata_connection as m:
+                    m.execute("UPDATE metadata SET " +   # nosec: trusted input
+                              "abs2prom=?, prom2abs=?, abs2meta=?, var_settings=?, conns=?",
+                              (abs2prom, prom2abs, abs2meta, var_settings_json, conns))
 
-    def record_iteration_driver(self, recording_requester, data, metadata):
+        self._started.add(recording_requester)
+
+    def record_iteration_driver(self, driver, data, metadata):
         """
         Record data and metadata from a Driver.
 
         Parameters
         ----------
-        recording_requester : object
+        driver : Driver
             Driver in need of recording.
         data : dict
             Dictionary containing desvars, objectives, constraints, responses, and System vars.
         metadata : dict
             Dictionary containing execution metadata.
         """
+        if not self._database_initialized:
+            raise RuntimeError(f"{driver.msginfo} attempted to record iteration to "
+                               f"'{self._filepath}', but database is not initialized;"
+                               " `run_model()`, `run_driver()`, or `final_setup()` "
+                               "must be called after adding a recorder.")
+
         if self.connection:
             outputs = data['output']
             inputs = data['input']
@@ -425,32 +511,38 @@ class SqliteRecorder(CaseRecorder):
                            inputs_text, outputs_text, residuals_text))
 
                 c.execute("INSERT INTO global_iterations(record_type, rowid, source) VALUES(?,?,?)",
-                          ('driver', c.lastrowid, recording_requester._get_name()))
+                          ('driver', c.lastrowid, driver._get_name()))
 
-    def record_iteration_problem(self, recording_requester, data, metadata):
+    def record_iteration_problem(self, problem, data, metadata):
         """
         Record data and metadata from a Problem.
 
         Parameters
         ----------
-        recording_requester : object
+        problem : Problem
             Problem in need of recording.
         data : dict
             Dictionary containing desvars, objectives, and constraints.
         metadata : dict
             Dictionary containing execution metadata.
         """
+        if not self._database_initialized:
+            raise RuntimeError(f"{problem.msginfo} attempted to record iteration to "
+                               f"'{self._filepath}', but database is not initialized;"
+                               " `run_model()`, `run_driver()`, or `final_setup()` "
+                               "must be called after adding a recorder.")
+
         if self.connection:
             outputs = data['output']
             inputs = data['input']
             residuals = data['residual']
 
-            driver = recording_requester.driver
-            if recording_requester.recording_options['record_derivatives'] and \
-                    driver._designvars and driver._responses:
+            driver = problem.driver
+            if problem.recording_options['record_derivatives'] and \
+               driver._designvars and driver._responses:
                 totals = data['totals']
             else:
-                totals = OrderedDict([])
+                totals = {}
             totals_array = dict_to_structured_array(totals)
             totals_blob = array_to_blob(totals_array)
 
@@ -465,29 +557,43 @@ class SqliteRecorder(CaseRecorder):
             inputs_text = json.dumps(inputs)
             residuals_text = json.dumps(residuals)
 
+            abs_err = data['abs']
+            rel_err = data['rel']
+
             with self.connection as c:
                 c = c.cursor()  # need a real cursor for lastrowid
 
                 c.execute("INSERT INTO problem_cases(counter, case_name, "
-                          "timestamp, success, msg, inputs, outputs, residuals, jacobian) "
-                          "VALUES(?,?,?,?,?,?,?,?,?)",
+                          "timestamp, success, msg, inputs, outputs, residuals, jacobian, "
+                          "abs_err, rel_err ) "
+                          "VALUES(?,?,?,?,?,?,?,?,?,?,?)",
                           (self._counter, metadata['name'],
                            metadata['timestamp'], metadata['success'], metadata['msg'],
-                           inputs_text, outputs_text, residuals_text, totals_blob))
+                           inputs_text, outputs_text, residuals_text, totals_blob,
+                           abs_err, rel_err))
 
-    def record_iteration_system(self, recording_requester, data, metadata):
+                c.execute("INSERT INTO global_iterations(record_type, rowid, source) VALUES(?,?,?)",
+                          ('problem', c.lastrowid, metadata['name']))
+
+    def record_iteration_system(self, system, data, metadata):
         """
         Record data and metadata from a System.
 
         Parameters
         ----------
-        recording_requester : System
+        system : System
             System in need of recording.
         data : dict
             Dictionary containing inputs, outputs, and residuals.
         metadata : dict
             Dictionary containing execution metadata.
         """
+        if not self._database_initialized:
+            raise RuntimeError(f"{system.msginfo} attempted to record iteration to "
+                               f"'{self._filepath}', but database is not initialized;"
+                               " `run_model()`, `run_driver()`, or `final_setup()` "
+                               "must be called after adding a recorder.")
+
         if self.connection:
             inputs = data['input']
             outputs = data['output']
@@ -513,26 +619,32 @@ class SqliteRecorder(CaseRecorder):
                            inputs_text, outputs_text, residuals_text))
 
                 # get the pathname of the source system
-                source_system = recording_requester.pathname
+                source_system = system.pathname
                 if source_system == '':
                     source_system = 'root'
 
                 c.execute("INSERT INTO global_iterations(record_type, rowid, source) VALUES(?,?,?)",
                           ('system', c.lastrowid, source_system))
 
-    def record_iteration_solver(self, recording_requester, data, metadata):
+    def record_iteration_solver(self, solver, data, metadata):
         """
         Record data and metadata from a Solver.
 
         Parameters
         ----------
-        recording_requester : Solver
+        solver : Solver
             Solver in need of recording.
         data : dict
             Dictionary containing outputs, residuals, and errors.
         metadata : dict
             Dictionary containing execution metadata.
         """
+        if not self._database_initialized:
+            raise RuntimeError(f"{solver.msginfo} attempted to record iteration to "
+                               f"'{self._filepath}', but database is not initialized;"
+                               " `run_model()`, `run_driver()`, or `final_setup()` "
+                               "must be called after adding a recorder.")
+
         if self.connection:
             abs = data['abs']
             rel = data['rel']
@@ -563,19 +675,19 @@ class SqliteRecorder(CaseRecorder):
                            abs, rel, inputs_text, outputs_text, residuals_text))
 
                 # get the pathname of the source system
-                source_system = recording_requester._system().pathname
+                source_system = solver._system().pathname
                 if source_system == '':
                     source_system = 'root'
 
                 # get solver type from SOLVER class attribute to determine the solver pathname
-                solver_type = recording_requester.SOLVER[0:2]
+                solver_type = solver.SOLVER[0:2]
                 if solver_type == 'NL':
                     source_solver = source_system + '.nonlinear_solver'
                 elif solver_type == 'LS':
                     source_solver = source_system + '.nonlinear_solver.linesearch'
                 else:
                     raise RuntimeError("Solver type '%s' not recognized during recording. "
-                                       "Expecting NL or LS" % recording_requester.SOLVER)
+                                       "Expecting NL or LS" % solver.SOLVER)
 
                 c.execute("INSERT INTO global_iterations(record_type, rowid, source) VALUES(?,?,?)",
                           ('solver', c.lastrowid, source_solver))
@@ -591,28 +703,32 @@ class SqliteRecorder(CaseRecorder):
         key : str, optional
             The unique ID to use for this data in the table.
         """
-        if self.connection:
-            json_data = json.dumps(model_viewer_data, default=make_serializable)
+        if self._record_metadata and self.metadata_connection:
+            json_data = json.dumps(model_viewer_data, default=default_noraise)
 
             # Note: recorded to 'driver_metadata' table for legacy/compatibility reasons.
             try:
-                with self.connection as c:
-                    c.execute("INSERT INTO driver_metadata(id, model_viewer_data) VALUES(?,?)",
+                with self.metadata_connection as m:
+                    m.execute("INSERT INTO driver_metadata(id, model_viewer_data) VALUES(?,?)",
                               (key, json_data))
             except sqlite3.IntegrityError:
-                print("Model viewer data has already has already been recorded for %s." % key)
+                print("Model viewer data has already been recorded for %s." % key)
 
-    def record_metadata_system(self, recording_requester):
+    def record_metadata_system(self, system, run_number=None):
         """
         Record system metadata.
 
         Parameters
         ----------
-        recording_requester : System
-            The System that would like to record its metadata.
+        system : System
+            The System for which to record metadata.
+        run_number : int or None
+            Number indicating which run the metadata is associated with.
+            None for the first run, 1 for the second, etc.
         """
-        if self.connection:
-            scaling_vecs, user_options = self._get_metadata_system(recording_requester)
+        if self._record_metadata and self.metadata_connection:
+
+            scaling_vecs, user_options = self._get_metadata_system(system)
 
             if scaling_vecs is None:
                 return
@@ -628,47 +744,58 @@ class SqliteRecorder(CaseRecorder):
                         pickle.dumps(values, self._pickle_version)
                 except Exception:
                     pickled_metadata = pickle.dumps(OptionsDictionary(), self._pickle_version)
-                    simple_warning("Trying to record option '%s' which cannot be pickled on system "
-                                   "%s. Set 'recordable' to False. Skipping recording options for "
-                                   "this system." % (key, recording_requester.msginfo))
+                    msg = f"Trying to record option '{key}' which cannot be pickled on this " \
+                          "system. Set option 'recordable' to False. Skipping recording options " \
+                          "for this system."
+                    issue_warning(msg, prefix=system.msginfo, category=CaseRecorderWarning)
 
-            path = recording_requester.pathname
+            path = system.pathname
             if not path:
                 path = 'root'
 
-            scaling_factors = sqlite3.Binary(scaling_factors)
-            pickled_metadata = sqlite3.Binary(pickled_metadata)
+            scaling_factors = sqlite3.Binary(zlib.compress(scaling_factors))
+            pickled_metadata = sqlite3.Binary(zlib.compress(pickled_metadata))
 
-            with self.connection as c:
-                # Because we can have a recorder attached to multiple Systems,
-                #   and because we are now recording System metadata recursively,
-                #   we can store System metadata multiple times. Need to ignore when that happens
-                #   so we don't get database errors. So use OR IGNORE
-                c.execute("INSERT OR IGNORE INTO system_metadata"
+            if run_number is None:
+                name = path
+            else:
+                name = META_KEY_SEP.join([path, str(run_number)])
+
+            with self.metadata_connection as m:
+                m.execute("INSERT INTO system_metadata"
                           "(id, scaling_factors, component_metadata) "
-                          "VALUES(?,?,?)", (path, scaling_factors, pickled_metadata))
+                          "VALUES(?,?,?)", (name, scaling_factors,
+                                            pickled_metadata))
 
-    def record_metadata_solver(self, recording_requester):
+    def record_metadata_solver(self, solver, run_number=None):
         """
         Record solver metadata.
 
         Parameters
         ----------
-        recording_requester : Solver
-            The Solver that would like to record its metadata.
+        solver : Solver
+            The Solver for which to record metadata.
+        run_number : int or None
+            Number indicating which run the metadata is associated with.
+            None for the first run, 1 for the second, etc.
         """
-        if self.connection:
-            path = recording_requester._system().pathname
-            solver_class = type(recording_requester).__name__
+        if self._record_metadata and self.metadata_connection:
+            path = solver._system().pathname
+            solver_class = type(solver).__name__
+
             if not path:
                 path = 'root'
+
             id = "{}.{}".format(path, solver_class)
 
-            solver_options = pickle.dumps(recording_requester.options, self._pickle_version)
+            if run_number is not None:
+                id = META_KEY_SEP.join([id, str(run_number)])
 
-            with self.connection as c:
-                c.execute("INSERT INTO solver_metadata(id, solver_options, solver_class) "
-                          "VALUES(?,?,?)", (id, sqlite3.Binary(solver_options), solver_class))
+            solver_options = zlib.compress(pickle.dumps(solver.options, self._pickle_version))
+
+            with self.metadata_connection as m:
+                m.execute("INSERT INTO solver_metadata(id, solver_options, solver_class)"
+                          " VALUES(?,?,?)", (id, sqlite3.Binary(solver_options), solver_class))
 
     def record_derivatives_driver(self, recording_requester, data, metadata):
         """
@@ -702,9 +829,19 @@ class SqliteRecorder(CaseRecorder):
         Shut down the recorder.
         """
         # close database connection
+        if self._record_metadata and self.metadata_connection and \
+                self.metadata_connection != self.connection:
+            self.metadata_connection.close()
+
         if self.connection:
             self.connection.close()
 
+        # sqlite close() does not always write until garbage collection occurs.
+        # If collection is not forced like this and a reader is immediately opened on
+        # the same file, it may find a 0-length or malformed db.
+        # See https://www.sqlite.org/c3ref/close.html for more info
+        gc.collect()
+
     def delete_recordings(self):
         """
         Delete all the recordings.
