@@ -6,40 +6,37 @@ ArmijoGoldsteinLS -- Like above, but terminates with the ArmijoGoldsteinLS condi
 
 """
 
-import sys
 import numpy as np
 
 from openmdao.core.analysis_error import AnalysisError
 from openmdao.solvers.solver import NonlinearSolver
 from openmdao.recorders.recording_iteration_stack import Recording
-from openmdao.utils.general_utils import simple_warning
+from openmdao.utils.om_warnings import issue_warning, SolverWarning
 
 
-def _print_violations(unknowns, lower, upper):
+def _print_violations(outputs, lower, upper):
     """
     Print out which variables exceed their bounds.
 
     Parameters
     ----------
-    unknowns : <Vector>
-        Vector containing the unknowns.
+    outputs : <Vector>
+        Vector containing the outputs.
     lower : <Vector>
         Vector containing the lower bounds.
     upper : <Vector>
         Vector containing the upper bounds.
     """
     start = end = 0
-    for name, val in unknowns._views_flat.items():
+    for name, val in outputs._abs_item_iter():
         end += val.size
-        if any(val > upper[start:end]):
-            print("'%s' exceeds upper bounds" % name)
-            print("  Val:", val)
-            print("  Upper:", upper[start:end], '\n')
+        if upper is not None and any(val > upper[start:end]):
+            msg = (f"'{name}' exceeds upper bounds\n  Val: {val}\n  Upper: {upper[start:end]}\n")
+            issue_warning(msg, category=SolverWarning)
 
-        if any(val < lower[start:end]):
-            print("'%s' exceeds lower bounds" % name)
-            print("  Val:", val)
-            print("  Lower:", lower[start:end], '\n')
+        if lower is not None and any(val < lower[start:end]):
+            msg = (f"'{name}' exceeds lower bounds\n  Val: {val}\n  Lower: {lower[start:end]}\n")
+            issue_warning(msg, category=SolverWarning)
 
         start = end
 
@@ -48,6 +45,11 @@ class LinesearchSolver(NonlinearSolver):
     """
     Base class for line search solvers.
 
+    Parameters
+    ----------
+    **kwargs : dict
+        Options dictionary.
+
     Attributes
     ----------
     _do_subsolve : bool
@@ -68,7 +70,7 @@ class LinesearchSolver(NonlinearSolver):
         **kwargs : dict
             Options dictionary.
         """
-        super(LinesearchSolver, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         # Parent solver sets this to control whether to solve subsystems.
         self._do_subsolve = False
         self._lower_bounds = None
@@ -78,7 +80,7 @@ class LinesearchSolver(NonlinearSolver):
         """
         Declare options before kwargs are processed in the init method.
         """
-        super(LinesearchSolver, self)._declare_options()
+        super()._declare_options()
         opt = self.options
         opt.declare(
             'bound_enforcement', default='scalar', values=['vector', 'scalar', 'wall'],
@@ -103,13 +105,13 @@ class LinesearchSolver(NonlinearSolver):
         depth : int
             depth of the current system (already incremented).
         """
-        super(LinesearchSolver, self)._setup_solvers(system, depth)
+        super()._setup_solvers(system, depth)
         if system._has_bounds:
-            abs2meta = system._var_abs2meta
+            abs2meta_out = system._var_abs2meta['output']
             start = end = 0
-            for abs_name, val in system._outputs._abs_val_iter():
+            for abs_name, val in system._outputs._abs_item_iter():
                 end += val.size
-                meta = abs2meta[abs_name]
+                meta = abs2meta_out[abs_name]
                 var_lower = meta['lower']
                 var_upper = meta['upper']
 
@@ -147,7 +149,7 @@ class LinesearchSolver(NonlinearSolver):
         """
         Enforce lower/upper bounds.
 
-        Modifies the vector of unknowns and the step.
+        Modifies the vector of outputs and the step.
 
         Parameters
         ----------
@@ -182,6 +184,11 @@ class BoundsEnforceLS(LinesearchSolver):
 
     Not so much a linesearch; just check the bounds and if they are violated, then pull back to a
     non-violating point and evaluate.
+
+    Parameters
+    ----------
+    **kwargs : dict
+        Options dictionary.
     """
 
     SOLVER = 'LS: BCHK'
@@ -190,7 +197,7 @@ class BoundsEnforceLS(LinesearchSolver):
         """
         Declare options before kwargs are processed in the init method.
         """
-        super(BoundsEnforceLS, self)._declare_options()
+        super()._declare_options()
         opt = self.options
 
         # Remove unused options from base options here, so that users
@@ -206,7 +213,7 @@ class BoundsEnforceLS(LinesearchSolver):
         system = self._system()
 
         u = system._outputs
-        du = system._vectors['output']['linear']
+        du = system._doutputs
 
         if not system._has_bounds:
             u += du
@@ -225,10 +232,8 @@ class BoundsEnforceLS(LinesearchSolver):
 
             self._run_apply()
             norm = self._iter_get_norm()
-            # With solvers, we want to record the norm AFTER
-            # the call, but the call needs to
-            # be wrapped in the with for stack purposes,
-            # so we locally assign  norm & norm0 into the class.
+
+            # Save the norm values in the context manager so they can also be recorded.
             rec.abs = norm
             rec.rel = norm / norm0
 
@@ -239,6 +244,11 @@ class ArmijoGoldsteinLS(LinesearchSolver):
     """
     Backtracking line search that terminates using the Armijo-Goldstein condition.
 
+    Parameters
+    ----------
+    **kwargs : dict
+        Options dictionary.
+
     Attributes
     ----------
     _analysis_error_raised : bool
@@ -250,13 +260,8 @@ class ArmijoGoldsteinLS(LinesearchSolver):
     def __init__(self, **kwargs):
         """
         Initialize all attributes.
-
-        Parameters
-        ----------
-        **kwargs : dict
-            Options dictionary.
         """
-        super(ArmijoGoldsteinLS, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
         self._analysis_error_raised = False
 
@@ -286,7 +291,7 @@ class ArmijoGoldsteinLS(LinesearchSolver):
         self.alpha = alpha = self.options['alpha']
 
         u = system._outputs
-        du = system._vectors['output']['linear']
+        du = system._doutputs
 
         self._run_apply()
         phi0 = self._line_search_objective()
@@ -324,7 +329,7 @@ class ArmijoGoldsteinLS(LinesearchSolver):
         """
         Declare options before kwargs are processed in the init method.
         """
-        super(ArmijoGoldsteinLS, self)._declare_options()
+        super()._declare_options()
         opt = self.options
         opt['maxiter'] = 5
         opt.declare('c', default=0.1, lower=0.0, upper=1.0, desc="Slope parameter for line of "
@@ -342,7 +347,6 @@ class ArmijoGoldsteinLS(LinesearchSolver):
         Perform the operations in the iteration loop.
         """
         self._analysis_error_raised = False
-        system = self._system()
 
         # Hybrid newton support.
         if self._do_subsolve and self._iter_count > 0:
@@ -419,7 +423,7 @@ class ArmijoGoldsteinLS(LinesearchSolver):
 
         system = self._system()
         u = system._outputs
-        du = system._vectors['output']['linear']  # Newton step
+        du = system._doutputs  # Newton step
 
         self._iter_count = 0
         phi = self._iter_initialize()
@@ -444,9 +448,7 @@ class ArmijoGoldsteinLS(LinesearchSolver):
 
                     phi = self._line_search_objective()
 
-                    # With solvers, we want to report the norm AFTER
-                    # the iter_execute call, but the i_e call needs to
-                    # be wrapped in the with for stack purposes.
+                    # Save the norm values in the context manager so they can also be recorded.
                     rec.abs = phi
                     rec.rel = phi / phi0
 
@@ -496,10 +498,11 @@ def _enforce_bounds_vector(u, du, alpha, lower_bounds, upper_bounds):
 
     # Find the largest amount a bound is violated
     # where positive means a bound is violated - i.e. the required d_alpha.
-    mask = du._data != 0
+    du_arr = du.asarray()
+    mask = du_arr != 0
     if mask.any():
-        abs_du_mask = np.abs(du._data[mask])
-        u_mask = u._data[mask]
+        abs_du_mask = np.abs(du_arr[mask])
+        u_mask = u.asarray()[mask]
 
         # Check lower bound
         if lower_bounds is not None:
@@ -554,7 +557,7 @@ def _enforce_bounds_scalar(u, du, alpha, lower_bounds, upper_bounds):
     # the step vector directly.
 
     # enforce bounds on step in-place.
-    u_data = u._data
+    u_data = u.asarray()
 
     # If u > lower, we're just adding zero. Otherwise, we're adding
     # the step required to get up to the lower bound.
@@ -568,9 +571,8 @@ def _enforce_bounds_scalar(u, du, alpha, lower_bounds, upper_bounds):
     change_upper = 0. if upper_bounds is None else np.minimum(u_data, upper_bounds) - u_data
 
     change = change_lower + change_upper
-
     u_data += change
-    du._data += change / alpha
+    du += change / alpha
 
 
 def _enforce_bounds_wall(u, du, alpha, lower_bounds, upper_bounds):
@@ -599,8 +601,8 @@ def _enforce_bounds_wall(u, du, alpha, lower_bounds, upper_bounds):
     # the step vector directly.
 
     # enforce bounds on step in-place.
-    u_data = u._data
-    du_data = du._data
+    u_data = u.asarray()
+    du_data = du.asarray()
 
     # If u > lower, we're just adding zero. Otherwise, we're adding
     # the step required to get up to the lower bound.
