@@ -2,7 +2,6 @@
 import os
 import re
 import sys
-import warnings
 import unittest
 from contextlib import contextmanager
 from fnmatch import fnmatchcase
@@ -14,9 +13,10 @@ from collections.abc import Iterable
 import numpy as np
 
 from openmdao.core.constants import INF_BOUND
-from openmdao.utils.om_warnings import issue_warning, _warn_simple_format, warn_deprecation
+from openmdao.utils.om_warnings import issue_warning, warn_deprecation
 from openmdao.utils.array_utils import shape_to_len
 
+
 # Certain command line tools can make use of this to allow visualization of models when errors
 # are present that would normally cause setup to abort.
 _ignore_errors = False
@@ -129,12 +129,7 @@ def simple_warning(msg, category=UserWarning, stacklevel=2):
     """
     warn_deprecation('simple_warning is deprecated. '
                      'Use openmdao.utils.om_warnings.issue_warning instead.')
-    old_format = warnings.formatwarning
-    warnings.formatwarning = _warn_simple_format
-    try:
-        warnings.warn(msg, category, stacklevel)
-    finally:
-        warnings.formatwarning = old_format
+    issue_warning(msg, stacklevel=stacklevel, category=category)
 
 
 def ensure_compatible(name, value, shape=None, indices=None):
