@@ -8,6 +8,7 @@ from numbers import Integral
 from itertools import zip_longest
 
 from openmdao.utils.general_utils import shape2tuple
+from openmdao.utils.array_utils import shape_to_len
 from openmdao.utils.om_warnings import issue_warning, OMDeprecationWarning
 
 
@@ -158,7 +159,7 @@ class Indexer(object):
             raise RuntimeError(f"Can't get indexed_src_shape of {self} because source shape "
                                "is unknown.")
         if self._flat_src:
-            return resolve_shape(np.product(self._src_shape))[self.flat()]
+            return resolve_shape(shape_to_len(self._src_shape))[self.flat()]
         else:
             return resolve_shape(self._src_shape)[self()]
 
@@ -172,7 +173,7 @@ class Indexer(object):
         int
             Size of flattened indices.
         """
-        return np.product(self.indexed_src_shape, dtype=int)
+        return shape_to_len(self.indexed_src_shape)
 
     def _check_ind_type(self, ind, types):
         if not isinstance(ind, types):
@@ -242,7 +243,7 @@ class Indexer(object):
             The resulting indices (always flat).
         """
         arr = self.shaped_array().ravel()
-        return arr[self.flat()]
+        return arr[subidxer.flat()]
 
     def set_src_shape(self, shape, dist_shape=None):
         """
@@ -260,12 +261,11 @@ class Indexer(object):
         Indexer
             Self is returned to allow chaining.
         """
-        if self._flat_src is None and shape is not None:
-            self._flat_src = len(shape2tuple(shape)) <= 1
-
         self._src_shape, self._dist_shape, = self._get_shapes(shape, dist_shape)
 
         if shape is not None:
+            if self._flat_src is None:
+                self._flat_src = len(self._src_shape) <= 1
             self._check_bounds()
 
         self._shaped_inst = None
@@ -284,14 +284,14 @@ class Indexer(object):
 
         shape = shape2tuple(shape)
         if self._flat_src:
-            shape = (np.product(shape),)
+            shape = (shape_to_len(shape),)
 
         if dist_shape is None:
             return shape, shape
 
         dist_shape = shape2tuple(dist_shape)
         if self._flat_src:
-            dist_shape = (np.product(dist_shape),)
+            dist_shape = (shape_to_len(dist_shape),)
 
         return shape, dist_shape
 
@@ -493,6 +493,8 @@ class ShapedSliceIndexer(Indexer):
         """
         super().__init__(flat_src)
         self._check_ind_type(slc, slice)
+        if slc.step is None:
+            slc = slice(slc.start, slc.stop, 1)
         self._slice = slc
 
     def __call__(self):
@@ -544,14 +546,26 @@ class ShapedSliceIndexer(Indexer):
         ndarray
             The index array.
         """
-        # use maxsize here since a shaped slice always has positive int start and stop
-        arr = np.arange(*self._slice.indices(sys.maxsize), dtype=int)
-        if flat:
-            return arr
-
-        if self._orig_shape is None:
-            return arr
-        return arr.reshape(self._orig_shape)
+        if len(self._src_shape) == 1:
+            # Case 1: Requested flat or nonflat indices but src_shape is None or flat
+            # return a flattened arange
+            slc = self._slice
+            if slc.stop is None and slc.step < 0:  # special case - neg step down to -1
+                return np.arange(self._src_shape[0], dtype=int)[slc]
+            else:
+                # use maxsize here since a shaped slice always has positive int start and stop
+                return np.arange(*slc.indices(sys.maxsize), dtype=int)
+        else:
+            src_size = shape_to_len(self._src_shape)
+            arr = np.arange(src_size, dtype=int).reshape(self._src_shape)[self._slice].ravel()
+            if flat:
+                # Case 2: Requested flattened indices of multidimensional array
+                # Return indices into a flattened src.
+                return arr
+            else:
+                # Case 3: Requested non-flat indices of multidimensional array
+                # This is never called within OpenMDAO
+                return np.unravel_index(arr, shape=self._src_shape)
 
     def flat(self, copy=False):
         """
@@ -592,7 +606,7 @@ class ShapedSliceIndexer(Indexer):
         if self._src_shape is not None:
             start = self._slice.start
             stop = self._slice.stop
-            sz = np.product(self._dist_shape)
+            sz = shape_to_len(self._dist_shape)
             if (start is not None and (start >= sz or start < -sz)
                     or (stop is not None and (stop > sz or stop < -sz))):
                 raise IndexError(f"{self._slice} is out of bounds of the source shape "
@@ -638,7 +652,10 @@ class SliceIndexer(ShapedSliceIndexer):
             return None
 
         slc = self._slice
-        if (slc.start is not None and slc.start < 0) or slc.stop is None or slc.stop < 0:
+        if slc.stop is None and slc.step < 0:  # special backwards indexing case
+            self._shaped_inst = \
+                ShapedSliceIndexer(slc)
+        elif (slc.start is not None and slc.start < 0) or slc.stop is None or slc.stop < 0:
             self._shaped_inst = \
                 ShapedSliceIndexer(slice(*self._slice.indices(self._src_shape[0])))
         else:
@@ -676,8 +693,7 @@ class SliceIndexer(ShapedSliceIndexer):
         """
         slc = self._slice
         if self._flat_src and slc.start is not None and slc.stop is not None:
-            step = 1 if slc.step is None else slc.step
-            return (len(range(slc.start, slc.stop, step)),)
+            return (len(range(slc.start, slc.stop, slc.step)),)
         return super().indexed_src_shape
 
 
@@ -775,7 +791,7 @@ class ShapedArrayIndexer(Indexer):
             The index array.
         """
         if flat:
-            arr = self._arr.flat[:]
+            arr = self._arr.ravel()
         else:
             arr = self._arr
         if copy:
@@ -797,15 +813,15 @@ class ShapedArrayIndexer(Indexer):
             The index into a flat array.
         """
         if copy:
-            return self._arr.flat[:].copy()
-        return self._arr.flat[:]
+            return self._arr.ravel().copy()
+        return self._arr.ravel()
 
     def _check_bounds(self):
         """
         Check that indices are within the bounds of the source shape.
         """
         if self._src_shape is not None and self._arr.size > 0:
-            src_size = np.product(self._dist_shape)
+            src_size = shape_to_len(self._dist_shape)
             amax = np.max(self._arr)
             ob = None
             if amax >= src_size or -amax < -src_size:
@@ -982,7 +998,7 @@ class ShapedMultiIndexer(Indexer):
         if self._src_shape is None:
             raise ValueError(f"Can't determine extent of array because source shape is not known.")
 
-        idxs = np.arange(np.product(self._src_shape), dtype=np.int32).reshape(self._src_shape)
+        idxs = np.arange(shape_to_len(self._src_shape), dtype=np.int32).reshape(self._src_shape)
 
         if flat:
             return idxs[self()].ravel()
@@ -1321,7 +1337,7 @@ class IndexMaker(object):
 
         if src_shape is not None:
             if flat_src:
-                src_shape = (np.product(src_shape),)
+                src_shape = (shape_to_len(src_shape),)
             idxer.set_src_shape(src_shape)
 
         return idxer
