@@ -19,10 +19,6 @@ else:
     except Exception:
         pass  # in case they're using an old version of coverage
 
-try:
-    import pkg_resources
-except ImportError:
-    pkg_resources = None
 
 import openmdao.utils.hooks as hooks
 from openmdao.visualization.n2_viewer.n2_viewer import n2
@@ -56,12 +52,13 @@ from openmdao.utils.coloring import _total_coloring_setup_parser, _total_colorin
     _partial_coloring_setup_parser, _partial_coloring_cmd, \
     _view_coloring_setup_parser, _view_coloring_exec
 from openmdao.utils.scaffold import _scaffold_setup_parser, _scaffold_exec
-from openmdao.utils.file_utils import _load_and_exec, _to_filename
+from openmdao.utils.file_utils import _load_and_exec, _to_filename, _iter_entry_points
 from openmdao.utils.entry_points import _list_installed_setup_parser, _list_installed_cmd, \
     split_ep, _compute_entry_points_setup_parser, _compute_entry_points_exec, \
         _find_plugins_setup_parser, _find_plugins_exec
+from openmdao.utils.reports_system import _list_reports_setup_parser, _list_reports_cmd, \
+    _view_reports_setup_parser, _view_reports_cmd
 from openmdao.utils.general_utils import ignore_errors
-from openmdao.utils.om_warnings import warn_deprecation
 
 
 def _n2_setup_parser(parser):
@@ -85,9 +82,6 @@ def _n2_setup_parser(parser):
                         help="create embeddable version.")
     parser.add_argument('--title', default=None,
                         action='store', dest='title', help='diagram title.')
-    parser.add_argument('--use_declare_partial_info', action='store_true',
-                        dest='use_declare_partial_info',
-                        help="ignored, now always true.")
 
 
 def _n2_cmd(options, user_args):
@@ -108,10 +102,6 @@ def _n2_cmd(options, user_args):
         def _noraise(prob):
             prob.model._raise_connection_errors = False
 
-        if options.use_declare_partial_info:
-            warn_deprecation("'--use_declare_partial_info' is now the"
-                             " default and the option is ignored.")
-
         def _viewmod(prob):
             n2(prob, outfile=options.outfile, show_browser=not options.no_browser,
                 title=options.title, embeddable=options.embeddable)
@@ -485,6 +475,8 @@ _command_map = {
                      'Generate total timings of calls to particular object instances.'),
     'list_installed': (_list_installed_setup_parser, _list_installed_cmd,
                        'List installed types recognized by OpenMDAO.'),
+    'list_reports': (_list_reports_setup_parser, _list_reports_cmd,
+                     'List available reports.'),
     'mem': (_mem_prof_setup_parser, _mem_prof_exec,
             'Profile memory used by OpenMDAO related functions.'),
     'mempost': (_mempost_setup_parser, _mempost_exec, 'Post-process memory profile output.'),
@@ -493,6 +485,7 @@ _command_map = {
                          'Compute coloring(s) for specified partial jacobians.'),
     'scaffold': (_scaffold_setup_parser, _scaffold_exec,
                  'Generate a simple scaffold for a component.'),
+    'scaling': (_scaling_setup_parser, _scaling_cmd, 'View driver scaling report.'),
     'summary': (_config_summary_setup_parser, _config_summary_cmd,
                 'Print a short top-level summary of the problem.'),
     'timing': (_timing_setup_parser, _timing_cmd, 'Collect timing information for all systems.'),
@@ -506,7 +499,8 @@ _command_map = {
     'view_dyn_shapes': (_view_dyn_shapes_setup_parser, _view_dyn_shapes_cmd,
                         'View the dynamic shape dependency graph.'),
     'view_mm': (_meta_model_parser, _meta_model_cmd, "View a metamodel."),
-    'scaling': (_scaling_setup_parser, _scaling_cmd, 'View driver scaling report.'),
+    'view_reports': (_view_reports_setup_parser, _view_reports_cmd,
+                     'View existing reports.'),
 }
 
 
@@ -540,32 +534,29 @@ def openmdao_cmd():
         parser_setup_func(subp)
         subp.set_defaults(executor=executor)
 
-    if pkg_resources is None:
-        print("\npkg_resources was not found, so no plugin entry points can be loaded.\n")
-    else:
-        # now add any plugin openmdao commands
-        epdict = {}
-        for ep in pkg_resources.iter_entry_points(group='openmdao_command'):
-            cmd, module, target = split_ep(ep)
-            # don't let plugins override the builtin commands
-            if cmd in _command_map:
-                raise RuntimeError("openmdao plugin command '{}' defined in {} conflicts with "
-                                   "builtin command '{}'.".format(cmd, module, cmd))
-            elif cmd in epdict:
-                raise RuntimeError("openmdao plugin command '{}' defined in {} conflicts with a "
-                                   "another plugin command defined in {}."
-                                   .format(cmd, module, epdict[cmd][1]))
-            epdict[cmd] = (ep, module)
+    # now add any plugin openmdao commands
+    epdict = {}
+    for ep in _iter_entry_points('openmdao_command'):
+        cmd, module, target = split_ep(ep)
+        # don't let plugins override the builtin commands
+        if cmd in _command_map:
+            raise RuntimeError("openmdao plugin command '{}' defined in {} conflicts with "
+                               "builtin command '{}'.".format(cmd, module, cmd))
+        elif cmd in epdict:
+            raise RuntimeError("openmdao plugin command '{}' defined in {} conflicts with a "
+                               "another plugin command defined in {}."
+                               .format(cmd, module, epdict[cmd][1]))
+        epdict[cmd] = (ep, module)
 
-        # sort commands by module and then by command name so commands from plugins will
-        # be grouped together.
-        for cmd, (ep, module) in sorted(epdict.items(), key=lambda x: x[1][1] + x[0]):
-            func = ep.load()
-            parser_setup_func, executor, help_str = func()
-            pkg = module.split('.', 1)[0]
-            subp = subs.add_parser(cmd, help='(%s plugin) ' % pkg + help_str)
-            parser_setup_func(subp)
-            subp.set_defaults(executor=executor)
+    # sort commands by module and then by command name so commands from plugins will
+    # be grouped together.
+    for cmd, (ep, module) in sorted(epdict.items(), key=lambda x: x[1][1] + x[0]):
+        func = ep.load()
+        parser_setup_func, executor, help_str = func()
+        pkg = module.split('.', 1)[0]
+        subp = subs.add_parser(cmd, help='(%s plugin) ' % pkg + help_str)
+        parser_setup_func(subp)
+        subp.set_defaults(executor=executor)
 
     # handle case where someone just runs `openmdao <script> [dashed-args]`
     args = [a for a in sys.argv[1:] if not a.startswith('-')]
